diff -rupN /usr/src/originales/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/base.c /usr/src/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/base.c
--- /usr/src/originales/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/base.c	2008-12-17 15:57:56.000000000 +0000
+++ /usr/src/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/base.c	2009-02-25 14:47:07.000000000 +0000
@@ -110,9 +110,21 @@ static struct proc_dir_entry *myproc;
 struct devtype {
 	char *desc;
 	unsigned int flags;
+	int ports;   /* Number of ports the card has */
+	int has_ec;  /* Does the card have an Echo Canceller module?
+			Often abused to tell if this card is a B410P or not */
 };
 
-static struct devtype wcb4xxp = { "Wildcard B410P", 0 };
+static struct devtype wcb4xxp = { "Wildcard B410P", .ports = 4, .has_ec = 1  };
+static struct devtype hfc2s =	{ "HFC-2S Junghanns.NET duoBRI PCI", .ports = 2 };
+static struct devtype hfc4s =	{ "HFC-4S Junghanns.NET quadBRI PCI", .ports = 4 };
+/* Junghanns octoBRI card. Needs testing. Remmed-out for now.
+   Also needs to have its PCI ID added.
+static struct devtype hfc8s =	{ "HFC-4S Junghanns.NET octoBRI PCI", .ports = 8 };
+*/
+static struct devtype hfc2s_OV ={ "OpenVox B200P", .ports = 2 };
+static struct devtype hfc4s_OV ={ "OpenVox B400P", .ports = 4 };
+static struct devtype hfc8s_OV ={ "OpenVox B800P", .ports = 8 }; 
 
 
 #if 0
@@ -600,13 +612,16 @@ static void ec_init(struct b4xxp *b4)
 	unsigned char b;
 	unsigned int i, j, mask;
 
+	if (! b4->has_ec)
+		return;
+
 /* Setup GPIO */
 	for (i=0; i < NUM_EC; i++) {
 		b = ec_read(b4, i, 0x1a0);
 
 		dev_info(b4->dev, "VPM %d/%d init: chip ver %02x\n", i, NUM_EC - 1, b);
 
-		for (j=0; j < 4; j++) {
+		for (j=0; j < b4->numspans; j++) {
 			ec_write(b4, i, 0x1a8 + j, 0x00);	/* GPIO out */
 			ec_write(b4, i, 0x1ac + j, 0x00);	/* GPIO dir */
 			ec_write(b4, i, 0x1b0 + j, 0x00);	/* GPIO sel */
@@ -625,7 +640,7 @@ static void ec_init(struct b4xxp *b4)
 		mask = 0x02020202 << (i * 4);
 
 /* Setup the tdm channel masks for all chips*/
-		for (j=0; j < 4; j++)
+		for (j=0; j < b4->numspans; j++)
 			ec_write(b4, i, 0x33 - j, (mask >> (j << 3)) & 0xff);
 
 /* Setup convergence rate */
@@ -863,7 +878,8 @@ static void hfc_assign_bchan_fifo_ec(str
 
 	if (first) {
 		first = 0;
-		dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
+		if (b4->has_ec)
+			dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
 	}
 
 	fifo = port * 2;
@@ -1192,7 +1208,7 @@ static void hfc_update_st_timers(struct 
 	int i, j;
 	struct b4xxp_span *s;
 
-	for (i=0; i < 4; i++) {
+	for (i=0; i < b4->numspans; i++) {
 		s = &b4->spans[i];
 
 		for (j=HFC_T1; j <= HFC_T3; j++) {
@@ -1394,12 +1410,22 @@ static void hfc_init_all_st(struct b4xxp
 
 	gpio = b4xxp_getreg8(b4, R_GPI_IN3);
 
-	for (i=0; i < 4; i++) {
+	for (i=0; i < b4->numspans; i++) {
 		s = &b4->spans[i];
 		s->parent = b4;
 		s->port = i;
 
-		nt = ((gpio & (1 << (i + 4))) == 0);		/* GPIO=0 = NT mode */
+		/* The way the Digium B410P card reads the NT/TE mode 
+		 * jumper is the oposite of how other HFC-4S cards do: 
+		 * - In B410P: GPIO=0: NT
+		 * - In Junghanns: GPIO=0: TE
+		 * The code below assumes has_ec <=> B410P
+		 */
+		if (b4->variety == "Wilcard B410P") /* Per card basis - More accurate than b4->has_ec - J.A.Deniz */
+			nt = ((gpio & (1 << (i + 4))) == 0);
+		else
+			nt = ((gpio & (1 << (i + 4))) != 0);
+
 		s->te_mode = !nt;
 
 		dev_info(b4->dev, "Port %d: %s mode\n", i + 1, (nt ? "NT" : "TE"));
@@ -1757,7 +1783,12 @@ static void b4xxp_init_stage1(struct b4x
  * set up the clock controller
  * we have a 24.576MHz crystal, so the PCM clock is 2x the incoming clock.
  */
-	b4xxp_setreg8(b4, R_BRG_PCM_CFG, 0x02);
+	
+	if (b4->variety == "Wilcard B410") /* Per card config.... J.A. Deniz */
+	    b4xxp_setreg8(b4, R_BRG_PCM_CFG,0x02);
+	else
+	    b4xxp_setreg8(b4, R_BRG_PCM_CFG, V_PCM_CLK);
+	    
 	flush_pci();
 
 	udelay(100);				/* wait a bit for clock to settle */
@@ -1835,6 +1866,91 @@ static void b4xxp_setleds(struct b4xxp *
 	ec_write(b4, 0, 0x1a8 + 3, val);
 }
 
+/* FIXME: unexplained bit magic ahead */
+/* This function handles LED blinking for cards that are not the Digium
+ * B410P
+ */
+static void b4xxp_update_leds_hfc(struct b4xxp *b4)
+{
+	int i, leds;
+	int led[4]; /* FIXME: Could an eight ports card work???? Better making its own function in per variety basis */
+	struct b4xxp_span *bspan;
+	
+	b4->blinktimer++;
+	for (i=0; i < b4->numspans; i++) {
+		bspan = &b4->spans[i];
+
+		if (bspan->span.flags & DAHDI_FLAG_RUNNING) {
+			if (bspan->span.alarms) {
+				if (b4->blinktimer == (led_fader_table[b4->alarmpos] >> 1))
+					led[i] = 2;
+
+				if (b4->blinktimer == 0xf)
+					led[i] = 0;
+			} else if (bspan->span.mainttimer || bspan->span.maintstat) {
+				if (b4->blinktimer == (led_fader_table[b4->alarmpos] >> 1))
+					led[i] = 1;
+
+				if (b4->blinktimer == 0xf)
+					led[i] = 0;
+			} else {
+				/* No Alarm */
+				led[i] = 1;
+			}
+		} 
+		else
+			led[i] = 0;
+	}
+
+	/* FIXME: Each card manage leds in a differnt way. So one section per card's variety
+	 * So we can support partially a card without led writing. Looking card status via software 
+	 * We might add leds signalling only for cards reported leds working ok
+	 * Better no leds than breaking system with wrong instructions - J.A. Deniz */
+	 
+	if (b4->variety == "OpenVox B400P") {
+		leds = ((led[0] > 0) << 0) | ((led[1] > 0) << 1) | 
+			((led[2] > 0) << 2) | ((led[3] > 0) << 3) | 
+			((led[0] & 1) << 4) | ((led[1] & 1) << 5) |
+			((led[2] & 1) << 6) | ((led[3] & 1) << 7); /* changed by james.zhu to set the leds of B400P */
+		b4xxp_setreg8(b4, R_GPIO_EN1, leds & 0x0f); 
+		b4xxp_setreg8(b4, R_GPIO_OUT1, leds  >> 4); 
+	}
+		
+	else if (b4->variety  == "OpenVox B200P") {
+		b4xxp_setreg8(b4, R_GPIO_SEL,0x20 | 0x10);
+		b4xxp_setreg8(b4, R_GPIO_EN1,0xf);
+		b4xxp_setreg8(b4, R_GPIO_OUT1,
+				(led[0] | (led[1] << 1) | (led[2] << 2) | 
+				 (led[3] << 3))); /*  Tested by J.A. Deniz */
+	} 
+
+	else if (b4->variety  == "HFC-2S Junghanns.NET duoBRI PCI") {
+		b4xxp_setreg8(b4, R_GPIO_SEL,0x20 | 0x10);
+		b4xxp_setreg8(b4, R_GPIO_EN1,0xf);
+		b4xxp_setreg8(b4, R_GPIO_OUT1,
+				(led[0] | (led[1] << 1) | (led[2] << 2) | 
+				 (led[3] << 3))); /* UNTESTED */
+	}
+
+	else if (b4->variety  == "HFC-4S Junghanns.NET quadBRI PCI") {
+		leds = ((led[0] > 0) << 0) | ((led[1] > 0) << 1) | 
+			((led[2] > 0) << 2) | ((led[3] > 0) << 3) | 
+			((led[0] & 1) << 4) | ((led[1] & 1) << 5) |
+			((led[2] & 1) << 6) | ((led[3] & 1) << 7); /* UNTESTED */
+		b4xxp_setreg8(b4, R_GPIO_EN1, leds & 0x0f); 
+		b4xxp_setreg8(b4, R_GPIO_OUT1, leds  >> 4); 
+	}
+
+	if (b4->blinktimer == 0xf) {
+		b4->blinktimer = -1;
+		b4->alarmpos++;
+		if (b4->alarmpos >= (sizeof(led_fader_table) / 
+					sizeof(led_fader_table[0])))
+			b4->alarmpos = 0;
+	}
+
+}
+
 static void b4xxp_set_span_led(struct b4xxp *b4, int span, unsigned char val)
 {
 	int shift, spanmask;
@@ -1847,11 +1963,22 @@ static void b4xxp_set_span_led(struct b4
 	b4xxp_setleds(b4, b4->ledreg);
 }
 
+
 static void b4xxp_update_leds(struct b4xxp *b4)
 {
 	int i;
 	struct b4xxp_span *bspan;
 
+	if (b4->variety == "OpenVox B800P")
+	    return; /* FIXME: No leds signalling for 8 ports cards */
+
+
+	 if (b4->variety != "Wildcard B410P") {
+		 /* Use the alternative function for non-Digium cards: */
+		 b4xxp_update_leds_hfc(b4);
+		 return;
+	 }
+
 	b4->blinktimer++;
 	for (i=0; i < b4->numspans; i++) {
 		bspan = &b4->spans[i];
@@ -1883,6 +2010,8 @@ static void b4xxp_update_leds(struct b4x
 	}
 }
 
+
+
 static int b4xxp_echocan(struct dahdi_chan *chan, int eclen)
 {
 	struct b4xxp *b4 = chan->pvt;
@@ -2138,7 +2267,7 @@ static void init_spans(struct b4xxp *b4)
 		bspan->span.close  = b4xxp_close;
 		bspan->span.ioctl = b4xxp_ioctl;
 		bspan->span.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit;
-		if (vpmsupport)
+		if (vpmsupport && b4->has_ec)
 			bspan->span.echocan = b4xxp_echocan;
 
 /* HDLC stuff */
@@ -2388,7 +2517,7 @@ static int b4xxp_proc_read_one(char *buf
 	}
 
 	strcat(sBuf, "\nPort states:\n");
-	for (i=0; i < 4; i++) {
+	for (i=0; i < b4->numspans; i++) {
 		int state;
 		char *x;
 		struct b4xxp_span *s = &b4->spans[i];
@@ -2483,6 +2612,7 @@ static int __devinit b4xx_probe(struct p
 /* card found, enabled and main struct allocated.  Fill it out. */
 	b4->magic = WCB4XXP_MAGIC;
 	b4->variety = dt->desc;
+	b4->has_ec = dt->has_ec;
 
 	b4->pdev = pdev;
 	b4->dev = &pdev->dev;
@@ -2512,7 +2642,8 @@ static int __devinit b4xx_probe(struct p
 */
 
 /* TODO: determine whether this is a 2, 4 or 8 port card */
-	b4->numspans = 4;
+	b4->numspans = dt->ports;
+	vpmsupport = dt->has_ec;
 	b4->syncspan = -1;		/* sync span is unknown */
 	if (b4->numspans > MAX_SPANS_PER_CARD) {
 		dev_err(b4->dev, "Driver does not know how to handle a %d span card!\n", b4->numspans);
@@ -2660,7 +2791,13 @@ static void __devexit b4xxp_remove(struc
 static struct pci_device_id b4xx_ids[] __devinitdata =
 {
 	{ 0xd161, 0xb410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&wcb4xxp },
-	{ 0, }
+	{ 0x1397, 0x08b4, 0x1397, 0xb520, 0, 0, (unsigned long)&hfc4s },
+	{ 0x1397, 0x08b4, 0x1397, 0xb556, 0, 0, (unsigned long)&hfc2s },
+	{ 0x1397, 0x08b4, 0x1397, 0xe884, 0, 0, (unsigned long)&hfc2s_OV },
+	{ 0x1397, 0x08b4, 0x1397, 0xe888, 0, 0, (unsigned long)&hfc4s_OV },
+	{ 0x1397, 0x08b4, 0x1397, 0xe998, 0, 0, (unsigned long)&hfc8s_OV },
+	{0, }
+
 };
 
 static struct pci_driver b4xx_driver = {
@@ -2719,7 +2856,7 @@ MODULE_PARM_DESC(timer_1_ms, "NT: msec t
 MODULE_PARM_DESC(timer_3_ms, "TE: msec to wait for link activation, NT: unused.");
 
 MODULE_AUTHOR("Digium Incorporated <support@digium.com>");
-MODULE_DESCRIPTION("B410P quad-port BRI module driver.");
+MODULE_DESCRIPTION("B410P & Similars multi-port BRI module driver.");
 MODULE_LICENSE("GPL");
 
 MODULE_DEVICE_TABLE(pci, b4xx_ids);
diff -rupN /usr/src/originales/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/wcb4xxp.h /usr/src/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/wcb4xxp.h
--- /usr/src/originales/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/wcb4xxp.h	2008-11-24 04:14:37.000000000 +0000
+++ /usr/src/dahdi-linux-2.1.0.4/drivers/dahdi/wcb4xxp/wcb4xxp.h	2009-02-25 12:06:20.000000000 +0000
@@ -452,6 +452,7 @@ struct b4xxp {
 	struct b4xxp_span spans[MAX_SPANS_PER_CARD];	/* Individual spans */
 	int order;				/* Order */
 	int flags;				/* Device flags */
+	int has_ec;				/* Has ECHO Cancel */
 	int master;				/* Are we master */
 	int ledreg;				/* copy of the LED Register */
 	unsigned int gpio;
